/**
 * @author Toru Nagashima
 * @copyright 2017 Toru Nagashima. All rights reserved.
 * See LICENSE file in root directory for full license.
 */
"use strict"

module.exports = {
    configs: { recommended: require("./recommended.json") },
    rules: require("./rules.js"),
}
